/***************************************************************************
 *   Copyright (C) 2007 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CUSTOM_PROG_H
#define CUSTOM_PROG_H

#include "progs/base/prog_group.h"

namespace CustomProgrammer
{
BEGIN_DECLARE_ENUM(Action)
  Read = 0, Erase, Program, Verify, BlankCheck, Run, Stop
END_DECLARE_ENUM_KEY_LABEL(Action)

//-----------------------------------------------------------------------------
class Config : public GenericConfig
{
public:
  Config() : GenericConfig("custom_programmer") {}
  static QString command(Action action);
  static void writeCommand(Action action, const QString &command);
};

//-----------------------------------------------------------------------------
class Base : public ::Programmer::Generic
{
public:
  Base(const ::Programmer::Group &group, const Device::Data *data) : Programmer::Generic(group, data) {}

  virtual ::Debugger::Base *debugger() { return 0; }
  virtual bool connectHardware() { return true; }
  virtual bool connectDevice() { return true; }
  virtual void disconnectHardware() {}
  virtual bool isTargetSelfPowered() const { return true; }
  virtual bool setTargetPowerOn(bool) { return true; }
  virtual bool isTargetPowerOn() const { return true; }

  virtual bool erase(const Device::MemoryRange &range) { return execute(Action::Erase, 0, &range); }
  virtual bool read(Device::Memory &memory, const Device::MemoryRange &range) { return execute(Action::Read, &memory, &range); }
  virtual bool program(const Device::Memory &memory, const Device::MemoryRange &range) { return execute(Action::Program, const_cast<Device::Memory *>(&memory), &range); }
  virtual bool verify(const Device::Memory &memory, const Device::MemoryRange &range) { return execute(Action::Verify, const_cast<Device::Memory *>(&memory), &range); }
  virtual bool blankCheck(const Device::MemoryRange &range) { return execute(Action::BlankCheck, 0, &range); }
  virtual bool run() { return execute(Action::Run, 0, 0); }
  virtual bool stop() { return execute(Action::Stop, 0, 0); }

private:
  bool execute(Action action, Device::Memory *memory, const Device::MemoryRange *range);
};

//-----------------------------------------------------------------------------
class Group : public ::Programmer::Group
{
public:
  virtual QString name() const { return "custom"; }
  virtual QString label() const { return i18n("Custom Programmer"); }
  virtual ::Programmer::Properties properties() const { return ::Programmer::Custom; }
  virtual ::Programmer::TargetPowerMode targetPowerMode() const { return ::Programmer::TargetSelfPowered; }
  virtual bool isPortSupported(PortType) const { return false; }
  virtual bool canReadVoltages() const { return false; }

protected:
  virtual void initSupported();
  virtual ::Programmer::Generic *createBase(const Device::Data *data) const { return new CustomProgrammer::Base(*this, data); }
  virtual ::Programmer::Hardware *createHardware(::Programmer::Base &, const ::Programmer::HardwareDescription &) const { return 0; }
  virtual ::Programmer::DeviceSpecific *createDeviceSpecific(::Programmer::Base &) const { return 0; }
  virtual ::Debugger::DeviceSpecific *createDebuggerDeviceSpecific(::Debugger::Base &) const { return 0; }
};

} // namespace

#endif
